package com.rapidminer.mapreduce;

/**
 * <p>A {@link Mapper} gets a chunk of the data and computes an interim
 * result independently to all over data chunks. This means that several
 * instances can run in parallel.</p>
 * 
 * <p>The interim results will be collected and sent to the reducer which
 * reduces all the interim results to a single result.</p>
 * 
 * @author Marcin Skirzynski
 *
 * @param <I>	the input for this concrete {@link Mapper}
 * @param <M>	the interim result which will be sent to the reducer 
 */
public interface Mapper<I, M> {
	
	/**
	 * Computes an interim result from the input.
	 * 
	 * @param input		the input for this {@link Mapper} which is a chunk of the whole data.
	 * @return			an interim result
	 */
	public M map(I input) throws MapReduceException;

}
