package com.rapidminer.mapreduce;

import java.util.Collection;

/**
 * <p>A {@link Reducer} receives a collection of interim results
 * and reduces these result to one single result.</p>
 * 
 * @author Marcin Skirzynski
 *
 * @param <M>	the type of the interim result
 * @param <O>	the type of the output
 */
public interface Reducer<M, O> {

	/**
	 * <p>Receives a collection of interim results which were
	 * computed by the {@link Mapper} and reduces them to one
	 * single value.</p> 
	 * 
	 * @param input	the interim results from the {@link Mapper}s
	 * @return		the result of the map and reduce process
	 */
	public O reduce(Collection<M> input) throws MapReduceException;
	
}
