package com.rapidminer.operator.performance;

import java.io.File;
import java.util.List;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeFile;
import com.rapidminer.parameter.ParameterTypeInt;

/**
 * Starts the measurement of the current process.
 * 
 * @author Marcin Skirzynski
 */
public class StartComputingPerformanceMeasurement extends Operator {
	
	/**
	 * Parameter key for the file to store the values
	 */
	private static final String PK_CSV_FILE = "csv_file";
	
	/**
	 * Parameter key for the pooling interval of the measurement
	 */
	private static final String PK_POOLING_INTERVAL = "interval";
	
	/**
	 * Output port for the perfomance object
	 */
	private OutputPort performanceOutput = getOutputPorts().createPort("performance");
	
	public StartComputingPerformanceMeasurement(OperatorDescription description) {
		super(description);
		
		getTransformer().addGenerationRule(performanceOutput, ComputingPerformance.class);
	}

	/**
	 * Starts a performance object and delivers it to the output port.
	 */
	@Override
	public void doWork() throws OperatorException {
		File file = getParameterAsFile(PK_CSV_FILE, true);
		int interval = getParameterAsInt(PK_POOLING_INTERVAL);
		
		ComputingPerformance performance = new ComputingPerformance(getProcess(), file, interval);
		performance.start();
		performanceOutput.deliver(performance);
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		
		types.add(new ParameterTypeFile(PK_CSV_FILE, "File where the performance values will be stored", "csv", false));
		types.add(new ParameterTypeInt(PK_POOLING_INTERVAL, "The interval in which the performance will be measured (in seconds)", 1, Integer.MAX_VALUE));
		
		return types;
	}
	
}
