package com.rapidminer.operator.preprocessing.association;

import com.rapidminer.operator.learner.associations.BooleanAttributeItem;
import com.rapidminer.operator.learner.associations.Item;

/**
 * Nearly an one-to-one copy of the {@link BooleanAttributeItem} since their is no
 * string constructor.
 * 
 * @author Marcin Skirzynski
 */
public class StringItem implements Item {
	private static final long serialVersionUID = -7963677912091349984L;

	private int frequency = 0;

	private String name;

	public StringItem(String item) {
		this.name = item;
	}
	
	public StringItem(String item, int frequency) {
		this(item);
		this.frequency = frequency;
	}

	public int getFrequency() {
		return this.frequency;
	}

	public void increaseFrequency() {
		this.frequency++;
	}

	public void increaseFrequency(double value) {
		this.frequency += value;
	}

	@Override
	public boolean equals(Object other) {
		if (!(other instanceof StringItem))
			return false;
		StringItem o = (StringItem)other;
		return (this.name.equals(o.name)) && (this.frequency == o.frequency);
	}
	
	@Override
	public int hashCode() {
		return this.name.hashCode() ^ Double.valueOf(this.frequency).hashCode();
	}
	
	public int compareTo(Item arg0) {
		Item comparer = arg0;
		// Collections.sort generates ascending order. Descending needed,
		// therefore invert return values!
		if (comparer.getFrequency() == this.getFrequency()) {
			return (-1 * this.name.compareTo(arg0.toString()));
		} else if (comparer.getFrequency() < this.getFrequency()) {
			return -1;
		} else {
			return 1;
		}
	}

	public String getName() {
		return name;
	}
	
	@Override
	public String toString() {
		return this.name;
	}

	public void increaseFrequency(int value) {
		frequency += value;
	}

}
