package com.rapidminer.operator.preprocessing.discretization;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.subspace.Interval;

/**
 * Class for testing purpose of the voptimal discretization. Is not used
 * in the extension anymore.
 * 
 * @author Marcin Skirzynski
 */
public class VOptimalDiscretization {
	
	public double computeVarianceSum(Attribute attribute, ExampleSet exampleSet, Collection<Interval> intervals) {
		if(!attribute.isNumerical()) {
			throw new IllegalArgumentException("No numerical attribute");
		}
		
		Map<Interval, Statistic> statistics = new HashMap<Interval, Statistic>();
		for(Interval interval : intervals) {
			if(interval.getAttributeName().equals(attribute.getName())) {
				statistics.put(interval, new Statistic());
			} else {
				System.err.println("Warning - interval passed for wrong attribute");
			}
		}
		
		for(Example example : exampleSet) {
			double value = example.getNumericalValue(attribute);
			Interval interval = getInterval(value, statistics.keySet());
			if(interval==null) {
				System.err.println("Warning - no interval found, skipping this example");
				continue;
			}
			Statistic statistic = statistics.get(interval);
			statistic.add(value);
		}
		
		double sum = 0.0;
		for(Statistic statistic : statistics.values()) {
			sum += statistic.size() * statistic.getVariance();
		}
		return sum;
	}
	
	private Interval getInterval(double value, Collection<Interval> intervals) {
		for(Interval interval : intervals) {
			if(interval.contains(value)) {
				return interval;
			}
		}
		return null;
	}
	
	/**
	 * Used to calculate the variance of the values.
	 * 
	 * @author Marcin Skirzynski
	 */
    private static class Statistic {
    	
    	private Collection<Double> values = new ArrayList<Double>();
    	
    	public void add(double value) {
    		values.add(value);
    	}
    	
    	public double getVariance() {
    		if(values.size()<=1)
    			return 0.0;
    		double sum = 0.0;
    		for(double value : values) {
    			sum += value;
    		}
    		double avg = sum/(double)values.size();

    		sum = 0.0;
    		for(double value : values) {
    			sum += Math.pow(value - avg, 2);
    		}
    		
    		return sum/(double)values.size();
    	}
    	
    	public int size() {
    		return values.size();
    	}
        
        @Override
        public String toString() {
        	return "<"+size()+","+getVariance()+">";
        }
    }

}
