package com.rapidminer.operator.preprocessing.discretization;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeDouble;
import com.rapidminer.subspace.Interval;
import com.rapidminer.tools.Ontology;

/**
 * Implementation for a test of the VOptiomal discretization. This isn't 
 * a fully implemented version of an discretization operator and was just used
 * for demonstration issues. Is not added as an operator anymore.
 * 
 * @author Marcin Skirzynski
 */
public class VOptimalDiscretizationOperator extends Operator {
	
	private static final String PK_FROM = "from";
	
	private static final String PK_TO = "to";
	
	private InputPort exampleSetInput = getInputPorts().createPort("example set", ExampleSet.class);
	
	private OutputPort exampleSetOutput = getOutputPorts().createPort("example set");

	public VOptimalDiscretizationOperator(OperatorDescription description) {
		super(description);
		getTransformer().addGenerationRule(exampleSetOutput, ExampleSet.class);
	}
	
	@Override
	public void doWork() throws OperatorException {
		ExampleSet exampleSet = exampleSetInput.getData();

		String attributeName = "att1";
		Attribute attribute = exampleSet.getAttributes().get(attributeName);
		
		double min = getParameterAsDouble(PK_FROM);
		double max = getParameterAsDouble(PK_TO);
		
		double l = min + 1;
		double h = max - 1;
		
		Interval i1 = new Interval(attributeName, min,l);
		Interval i4 = new Interval(attributeName, h, max);
		
		Attribute splitPointAttribute = AttributeFactory.createAttribute("Split Point", Ontology.REAL);
		Attribute wVarianceSumAttribute = AttributeFactory.createAttribute("Weighted Variance Sum", Ontology.REAL);
		
		MemoryExampleTable table = new MemoryExampleTable(splitPointAttribute, wVarianceSumAttribute);
		
		
		VOptimalDiscretization vopt = new VOptimalDiscretization();
//		
//		Interval i1a = new Interval(attributeName, 1.0, 6.0);
//		Interval i1b = new Interval(attributeName, 6.0, 18.0);
//		Collection<Interval> intervals1 = new ArrayList<Interval>();
//		intervals1.add(i1a);
//		intervals1.add(i1b);
//		
//		Interval i2a = new Interval(attributeName, 1.0, 6.1);
//		Interval i2b = new Interval(attributeName, 6.1, 18.0);
//		Collection<Interval> intervals2 = new ArrayList<Interval>();
//		intervals2.add(i2a);
//		intervals2.add(i2b);
//		
//		double var1 = vopt.computeVarianceSum(attribute, exampleSet, intervals1);
//		double var2 = vopt.computeVarianceSum(attribute, exampleSet, intervals2);
//		
//		System.out.println("Var1: " + var1);
//		System.out.println("Var2: " + var2);
		
		
		for(double i=0.5; i<(h-l); i+=0.1) {
			Interval i2 = new Interval(attributeName, l, l+i);
			Interval i3 = new Interval(attributeName, l+i, h);
			
			Collection<Interval> intervals = new ArrayList<Interval>();
			intervals.add(i1);
			intervals.add(i2);
			intervals.add(i3);
			intervals.add(i4);
			
			double varSum = vopt.computeVarianceSum(attribute, exampleSet, intervals);
			
			DoubleArrayDataRow row = new DoubleArrayDataRow(new double[] {l+i, varSum});
			table.addDataRow(row);
			
		}
		
		exampleSetOutput.deliver(table.createExampleSet());
		
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeDouble(PK_FROM, "From left", Integer.MIN_VALUE, Integer.MAX_VALUE,2));
		types.add(new ParameterTypeDouble(PK_TO, "To right", Integer.MIN_VALUE, Integer.MAX_VALUE, 18));
		return types;
	}

}
