package com.rapidminer.operator.preprocessing.transformation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.rapidminer.operator.IOObjectCollection;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.subspace.Interval;
import com.rapidminer.subspace.MinimalDescription;
import com.rapidminer.subspace.MinimalDescriptionCollection;
import com.rapidminer.subspace.Region;

/**
 * Takes a collection of descriptions and builds the highest possible
 * clustering for all found intervals.
 * 
 * @author Marcin Skirzynski
 */
public class ConstructHighDimensionalSubspaceCluster extends Operator {
	
	private InputPort minimalDescriptionInput = getInputPorts().createPort("minimal descriptions", IOObjectCollection.class);
	
	private OutputPort passthrough = getOutputPorts().createPassThroughPort("minimal descriptions");
	
	private OutputPort minimalDescriptionOutput = getOutputPorts().createPort("high dimensional");

	public ConstructHighDimensionalSubspaceCluster(OperatorDescription description) {
		super(description);
		getTransformer().addGenerationRule(minimalDescriptionOutput, IOObjectCollection.class);
		getTransformer().addPassThroughRule(minimalDescriptionInput, passthrough);
	}
	
	@Override
	public void doWork() throws OperatorException {
		IOObjectCollection<MinimalDescriptionCollection> input = minimalDescriptionInput.getData();
		
		Set<Interval> intervals = new HashSet<Interval>(); 
		for(MinimalDescriptionCollection descriptions : input.getObjects()) {
			if(descriptions.getDimension()>1) 
				continue;
			for(MinimalDescription description : descriptions.getObjects()) {
				if(description.getRegions().size()==1) {
					Region region = description.getRegions().iterator().next();
					if(region.getIntervals().size()>1) {
						System.err.println("Warning ... region with more than one interval... skipping");
						continue;
					}
					Interval interval = region.getIntervals().iterator().next();
					intervals.add(interval);
				}
			}
		}
		
		Region region = new Region(intervals);
		
		MinimalDescription singleDescription = new MinimalDescription(region);
		List<MinimalDescription> singleDescriptionList = new ArrayList<MinimalDescription>();
		singleDescriptionList.add(singleDescription);
		
		IOObjectCollection<IOObjectCollection<MinimalDescription>> result = 
				new IOObjectCollection<IOObjectCollection<MinimalDescription>>();
		
		result.add(new MinimalDescriptionCollection(singleDescriptionList, region.getAttributes().size()));
		
		minimalDescriptionOutput.deliver(result);
		passthrough.deliver(input);
	}

}
