package com.rapidminer.operator.preprocessing.transformation;

import java.util.Iterator;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.IOObjectCollection;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.subspace.Interval;
import com.rapidminer.subspace.MinimalDescription;
import com.rapidminer.subspace.MinimalDescriptionCollection;
import com.rapidminer.subspace.Region;
import com.rapidminer.tools.Ontology;

/**
 * Transforms the 1d and 2d descriptions to a example set
 * to plot them externally.
 * 
 * @author Marcin Skirzynski
 */
public class MinimalDescription2ExampleSet extends Operator {

	private InputPort minimalDescriptionsInput = getInputPorts().createPort("minimal descriptions", IOObjectCollection.class);
	private OutputPort minimalDescriptionsOutput = getOutputPorts().createPassThroughPort("minimal descriptions");

	private OutputPort exampleSet1dOutput = getOutputPorts().createPort("1d example sets"); 
	private OutputPort exampleSet2dOutput = getOutputPorts().createPort("2d example sets"); 
	
	public MinimalDescription2ExampleSet(OperatorDescription description) {
		super(description);
		getTransformer().addPassThroughRule(minimalDescriptionsInput, minimalDescriptionsOutput);
	}

	@Override
	public void doWork() throws OperatorException {
		IOObjectCollection<MinimalDescriptionCollection> descriptions = minimalDescriptionsInput.getData();
		
		Attribute name = AttributeFactory.createAttribute("attribute", Ontology.NOMINAL);
		Attribute from = AttributeFactory.createAttribute("from", Ontology.REAL);
		Attribute to = AttributeFactory.createAttribute("to", Ontology.REAL);
		
		Attribute name1 = AttributeFactory.createAttribute("attribute1", Ontology.NOMINAL);
		Attribute from1 = AttributeFactory.createAttribute("from1", Ontology.REAL);
		Attribute to1 = AttributeFactory.createAttribute("to1", Ontology.REAL);
		
		Attribute name2 = AttributeFactory.createAttribute("attribute2", Ontology.NOMINAL);
		Attribute from2 = AttributeFactory.createAttribute("from2", Ontology.REAL);
		Attribute to2 = AttributeFactory.createAttribute("to2", Ontology.REAL);
		
		MemoryExampleTable table1d = new MemoryExampleTable(name, from, to);
		MemoryExampleTable table2d = new MemoryExampleTable(name1, from1, to1, name2, from2, to2);
		
		for(MinimalDescriptionCollection mdCollection : descriptions.getObjects()) {
			for(MinimalDescription description : mdCollection.getObjects()) {
				for(Region region : description.getRegions()) {
	
					int dimensionality = region.getIntervals().size();
					if(dimensionality==1) {
						double[] data = new double[3];
						Interval interval = region.getIntervals().iterator().next();
						
						data[0] = name.getMapping().mapString(interval.getAttributeName());
						data[1] = interval.getLeft();
						data[2] = interval.getRight();
						
						table1d.addDataRow(new DoubleArrayDataRow(data));
					} else if(dimensionality==2) {
						double[] data = new double[6];
						Iterator<Interval> iter = region.getIntervals().iterator();
						Interval interval1 = iter.next();
						Interval interval2 = iter.next();
						
						data[0] = name1.getMapping().mapString(interval1.getAttributeName());
						data[1] = interval1.getLeft();
						data[2] = interval1.getRight();
						
						data[3] = name2.getMapping().mapString(interval2.getAttributeName());
						data[4] = interval2.getLeft();
						data[5] = interval2.getRight();
						
						table2d.addDataRow(new DoubleArrayDataRow(data));
					}
				}
			}
		}
		
		minimalDescriptionsOutput.deliver(descriptions);
		exampleSet1dOutput.deliver(table1d.createExampleSet());
		exampleSet2dOutput.deliver(table2d.createExampleSet());
	}
	
}
