package com.rapidminer.operator.preprocessing.transformation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.IOObjectCollection;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.features.selection.AbstractFeatureSelection;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.subspace.MinimalDescription;
import com.rapidminer.subspace.MinimalDescriptionCollection;

/**
 * Prunes all attributes which are not involved in a given Subspace Clustering.
 * 
 * @author Marcin Skirzynski
 */
public class PruneAttributesBySubspaceCluster extends AbstractFeatureSelection {
	
	private InputPort minimalDescriptionsInput = getInputPorts().createPort("minimal descriptions", IOObjectCollection.class);

	public PruneAttributesBySubspaceCluster(OperatorDescription description) {
		super(description);
	}

	@Override
	public ExampleSet apply(ExampleSet exampleSet) throws OperatorException {
		@SuppressWarnings("unchecked")
		IOObjectCollection<MinimalDescriptionCollection> mds = minimalDescriptionsInput.getData(IOObjectCollection.class);
		
		
		Set<String> attributeNames = new HashSet<String>();
		for(MinimalDescriptionCollection mdc : mds.getObjects()) {
			for(MinimalDescription md : mdc.getObjects()) {
				attributeNames.addAll(md.getSubspace().getAttributeNames());
			}
		}
		
		Collection<Attribute> removeSet = new ArrayList<Attribute>();
		for(Attribute attribute : exampleSet.getAttributes()) {
			if(!attributeNames.contains(attribute.getName())) {
				removeSet.add(attribute);
			}
		}
		
		for(Attribute attribute : removeSet) {
			exampleSet.getAttributes().remove(attribute);
		}
		
		return exampleSet;
	}

}
