package com.rapidminer.operator.preprocessing.transformation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeBoolean;
import com.rapidminer.subspace.Subspace;
import com.rapidminer.subspace.SubspaceCluster;
import com.rapidminer.subspace.SubspaceClustering;

/**
 * Prunes all subspaces which have only one cluster in it.
 * 
 * @author Marcin Skirzynski
 *
 */
public class PruneSingleCluster extends Operator {
	
	public static final String PARAMETER_SINGLE_UNIT = "prune-just-single-units";

	private InputPort subspaceClusteringInput = getInputPorts().createPort("subspace clustering", SubspaceClustering.class);
	
	private OutputPort prunedOutput = getOutputPorts().createPort("pruned");
	private OutputPort subspaceClusteringOutput = getOutputPorts().createPassThroughPort("subspace clustering");
	
	public PruneSingleCluster(OperatorDescription description) {
		super(description);
		
		getTransformer().addPassThroughRule(subspaceClusteringInput, subspaceClusteringOutput);
	}
	
	@Override
	public void doWork() throws OperatorException {
		SubspaceClustering clustering = subspaceClusteringInput.getData(SubspaceClustering.class);
		
		boolean pruneJustSingleUnits = getParameterAsBoolean(PARAMETER_SINGLE_UNIT);
		
		Collection<SubspaceCluster> multipleCluster = new ArrayList<SubspaceCluster>();
		for(Subspace subspace : clustering.getSubspaces()) {
			Set<SubspaceCluster> clusters = clustering.getCluster(subspace);
			if(clusters.size()>1) {
				multipleCluster.addAll(clusters);
			} else if(pruneJustSingleUnits) {
				SubspaceCluster cluster = clusters.iterator().next();
				if(!cluster.isSingleUnitCluster())
					multipleCluster.add(cluster);
			}
		}
		
		prunedOutput.deliver(new SubspaceClustering(multipleCluster));
		subspaceClusteringOutput.deliver(clustering);
	}
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		
		types.add(new ParameterTypeBoolean(PARAMETER_SINGLE_UNIT, "Just single cluster which are derived from a single unit will be pruned", false, true));
		
		return types;
	}

}
