package com.rapidminer.operator.preprocessing.transformation;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DataRow;
import com.rapidminer.example.table.DataRowFactory;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.tools.Ontology;

/**
 * Renames all genes to more handy names and returns a mapping.
 * 
 * @author Marcin Skirzynski
 *
 */
public class RenameGenes extends Operator {

	private InputPort exampleSetInput = getInputPorts().createPort("example set", ExampleSet.class);
	
	private OutputPort exampleSetOutput = getOutputPorts().createPort("example set");
	
	private OutputPort mappingOutput = getOutputPorts().createPort("mapping");
	
	public RenameGenes(OperatorDescription description) {
		super(description);
	}
	
	@Override
	public void doWork() throws OperatorException {
		
		ExampleSet exampleSet = exampleSetInput.getData(ExampleSet.class);
		
		Attribute oldNameAttr = AttributeFactory.createAttribute("old_name", Ontology.NOMINAL);
		Attribute newNameAttr = AttributeFactory.createAttribute("new_name", Ontology.NOMINAL);
		
		Attribute[] attributes = new Attribute[] {oldNameAttr, newNameAttr};

		MemoryExampleTable table = new MemoryExampleTable(attributes);
		DataRowFactory dataRowFactory = new DataRowFactory(DataRowFactory.TYPE_DOUBLE_ARRAY, '.');
		
		int index = 1;
		for(Attribute attribute : exampleSet.getAttributes()) {
			
			String oldName = attribute.getName();
			String newName = "G" + String.format("%02d", index++);
			attribute.setName(newName);

			DataRow row = dataRowFactory.create(new String[] {oldName, newName}, attributes);
			table.addDataRow(row);
			
		}
		
		exampleSetOutput.deliver(exampleSet);
		mappingOutput.deliver(table.createExampleSet());
		
	}

}
