package com.rapidminer.operator.preprocessing.transformation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DataRow;
import com.rapidminer.example.table.DataRowFactory;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.tools.Ontology;

/**
 * Transforms the data set to a transaction database which can be used
 * as input for frequent itemset mining.
 * 
 * @author Marcin Skirzynski
 *
 */
public class Transactionizer extends ExampleSetTransformationOperator{
	
	public static final String BOOLEAN_PREFIX = "__BOOL__";

	public Transactionizer(OperatorDescription description) {
		super(description);
	}

	@Override
	public ExampleSet apply(ExampleSet exampleSet) throws OperatorException {
		
		// create table
		Map<String, Attribute> attributes = new HashMap<String, Attribute>();
		exampleSet.recalculateAllAttributeStatistics();
		for( Attribute attribute : exampleSet.getAttributes()) {

			Collection<String> values = new ArrayList<String>();
			if(attribute.getValueType()==Ontology.BINOMINAL) {
				values.add(attribute.getMapping().getPositiveString());
				values.add(attribute.getMapping().getNegativeString());
			} else {
				values.addAll(attribute.getMapping().getValues());
			}
											
			for( String value : values ) {
				String newName = getAttributeNameForValue(attribute, value);
				Attribute newAttribute = AttributeFactory.createAttribute(newName, Ontology.BINOMINAL);
				newAttribute.getMapping().mapString("false");
				newAttribute.getMapping().mapString("true");
				attributes.put(newName, newAttribute);
			}
		}
		
		MemoryExampleTable table = new MemoryExampleTable(new ArrayList<Attribute>(attributes.values()));
		int size = attributes.size();
		
		DataRowFactory dataRowFactory = new DataRowFactory(DataRowFactory.TYPE_BOOLEAN_SPARSE_ARRAY, '.');
		for( Example example : exampleSet ) {
			DataRow row = dataRowFactory.create(size);
			for( Attribute attribute : exampleSet.getAttributes() ) {
				String value = example.getNominalValue(attribute);
				if(value.equals(Attribute.MISSING_NOMINAL_VALUE))
					continue;
				String newName = getAttributeNameForValue(attribute, value);
				Attribute newAttribute = attributes.get(newName);
				row.set(newAttribute, 1);
			}
			table.addDataRow(row);
		
		}
		
		return table.createExampleSet();
	}
	
	/**
	 * Creates a unique name for the attribute for the given value. 
	 */
	private String getAttributeNameForValue(Attribute attribute, String value) {
		String name = attribute.getName()+"_"+value;
		if(attribute.getValueType()==Ontology.BINOMINAL)
			name = BOOLEAN_PREFIX + name;
		return name;
	}
	
}
