package com.rapidminer.subspace;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Class for a depth-first search for connected units.
 * 
 * @author Marcin Skirzynski
 */
public class ClusterSearch {
	
	/**
	 * Finds Subspace cluster for the given units, which have to 
	 * be in the same subspace.
	 */
	public Collection<SubspaceCluster> find(Set<Unit> units) {
		if(!haveSameSubspace(units))
			throw new IllegalArgumentException("Can not find cluster for units not in the same subspace");
		
		Collection<SubspaceCluster> cluster = new ArrayList<SubspaceCluster>();
	
		Set<Unit> visited = new HashSet<Unit>();
		for( Unit unit : units ) {
			if(visited.contains(unit))
				continue;
			Set<Unit> connected = new HashSet<Unit>();
			connected.add(unit);
			dfs(unit, units, visited, connected);
			cluster.add(new SubspaceCluster(unit.getSubspace(), connected));
		}
		
		return cluster;
	}

	/**
	 * Starts a depth-first search for connected units.
	 */
	private void dfs(Unit unit, Set<Unit> units, Set<Unit> visited, Set<Unit> connected) {
		visited.add(unit);
		for(Unit u : units ) {
			if(visited.contains(u))
				continue;
			if(unit.hasCommonFace(u)) {
				connected.add(u);
				dfs(u, units, visited, connected);
			}
		}
	}
	
	/**
	 * Sanity check:<br/>
	 * Checks if all units in the given set are in the same
	 * subspace.
	 */
	private boolean haveSameSubspace(Set<Unit> units) {
		if(units.isEmpty())
			return true;
		Iterator<Unit> iter = units.iterator();
		Unit unit = iter.next();
		Subspace subspace = unit.getSubspace();
		while(iter.hasNext()) {
			unit = iter.next();
			if(!unit.getSubspace().equals(subspace))
				return false;
		}
		return true;
	}
	
}
