package com.rapidminer.subspace;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.rapidminer.operator.learner.associations.FrequentItemSet;
import com.rapidminer.operator.learner.associations.Item;
import com.rapidminer.operator.preprocessing.transformation.Transactionizer;

/**
 * Parses a {@link Item} from a {@link FrequentItemSet} to an {@link Interval} object.
 * 
 * @author Marcin Skirzynski
 */
public class ItemParser {
	
	public static final String SEPERATOR = "_";
	
	private static final String REGEX = "(-?[0-9]+\\.?[0-9]*|-?\\D) - (-?[0-9]+\\.?[0-9]*|-?\\D)";
	
    private static Locale FORMAT_LOCALE = Locale.US;

    /** Used for formatting values in the {@link #formatNumber(double)} method. */
    private static NumberFormat NUMBER_FORMAT = NumberFormat.getInstance(FORMAT_LOCALE);
    
    public static Pattern PATTERN = Pattern.compile(REGEX);
	
    /**
     * Parses a {@link Item} from a {@link FrequentItemSet} to an {@link Interval} object.
     */
	public Interval parse(Item item) throws ParseException {
		if(isBoolean(item)) {
			throw new IllegalArgumentException("This is a boolean item and can not be parsed to an item - check with 'isBoolean'-method before.");
		}
		
		String sitem = item.toString();
		int splitIndex = sitem.lastIndexOf('_');
		String attributeName = sitem.substring(0, splitIndex);
		String intervalString = sitem.substring(splitIndex+1, sitem.length());
		
		Matcher matcher = PATTERN.matcher(intervalString);
		
		if( !matcher.find() )
			throw new IllegalArgumentException("Can not parse item string '"+intervalString+"'");

		String group1 = matcher.group(1);
		String group2 = matcher.group(2);
		
		double left = NUMBER_FORMAT.parse(group1).doubleValue();
		double right = NUMBER_FORMAT.parse(group2).doubleValue();
		
		return new Interval(attributeName, left, right);
	}
	
	public boolean isBoolean(Item item) {
		return item.toString().startsWith(Transactionizer.BOOLEAN_PREFIX);
	}
	
	public String getBooleanAttributeName(Item item) {
		return item.toString().substring(Transactionizer.BOOLEAN_PREFIX.length());
	}

}
