package com.rapidminer.subspace;

import java.io.Serializable;
import java.util.Collections;
import java.util.Set;

/**
 * A subspace is simply a set of attributes.
 * 
 * @author Marcin Skirzynski
 *
 */
public class Subspace implements Serializable{
	private static final long serialVersionUID = 5655237420005986454L;

	/**
	 * The attribute names for this subspace. 
	 */
	private final Set<String> attributeNames;

	/**
	 * Creates an subspace with the given attribute names. 
	 */
	public Subspace(Set<String> attributeNames) {
		this.attributeNames = attributeNames;
	}

	/**
	 * Returns an unmodifiable set of the attribute names.
	 */
	public Set<String> getAttributeNames() {
		return Collections.unmodifiableSet(attributeNames);
	}
	
	/**
	 * Returns the number of attributes for this subspace, i.e. the dimensionality.
	 */
	public int size() {
		return attributeNames.size();
	}

	/**
	 * A simple string representation of this subspace.
	 */
	@Override
	public String toString() {
		return "Subspace ["+ attributeNames + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((attributeNames == null) ? 0 : attributeNames.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Subspace other = (Subspace) obj;
		if (attributeNames == null) {
			if (other.attributeNames != null)
				return false;
		} else if (!attributeNames.equals(other.attributeNames))
			return false;
		return true;
	}
	
	
	
}
