package com.rapidminer.subspace;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.rapidminer.operator.ResultObjectAdapter;

/**
 * A subspace clustering consists of a set of subspace and the cluster contained
 * in this subspaces. The cluster consists of connected units, but not minimal
 * descriptions is calculated.
 * 
 * @author Marcin Skirzynski
 */
public class SubspaceClustering extends ResultObjectAdapter {
	private static final long serialVersionUID = 690592606744826762L;

	/**
	 * All found subspace cluster in all subspaces.
	 */
	private final Set<SubspaceCluster> clusters;

	/**
	 * A mapping for all subspaces which include cluster
	 * to all the cluster found in there.
	 */
	private final Map<Subspace, Set<SubspaceCluster>> subspaces;

	/**
	 * Constructs an immutable subspace clustering.
	 */
	public SubspaceClustering(Collection<SubspaceCluster> clusters) {
		this.clusters = Collections.unmodifiableSet(new HashSet<SubspaceCluster>(clusters));
		this.subspaces = new HashMap<Subspace, Set<SubspaceCluster>>();
		for(SubspaceCluster cluster : clusters) {
			Subspace subspace = cluster.getSubspace();
			if(!subspaces.containsKey(subspace))
				subspaces.put(subspace, new HashSet<SubspaceCluster>());
			this.subspaces.get(subspace).add(cluster);
		}
		System.out.println(clusters.size() + " Subspace Cluster created in " + subspaces.size() + " subspaces");
	}
	
	/**
	 * Returns an unmodifiable set of all clusters in this clustering.
	 */
	public Set<SubspaceCluster> getClusters() {
		return clusters;
	}
	
	/**
	 * Returns all subspaces where clusters were found.
	 */
	public Set<Subspace> getSubspaces() {
		return Collections.unmodifiableSet(subspaces.keySet());
	}
	
	/**
	 * Returns all clusters for a specific subspace
	 */
	public Set<SubspaceCluster> getCluster(Subspace subspace) {
		return Collections.unmodifiableSet(subspaces.get(subspace));
	}

}
