package com.rapidminer.mapreduce;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Random;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;


public class LocalMapReduceTest implements MapperFactory<Integer[], Integer>, ReducerFactory<Integer, Integer>{

	@Before
	public void setUp() throws Exception {
	}

	@Test
	public void test() throws MapReduceException {
		
		MapReduceExecutor<Integer[], Integer, Integer> mr = new MapReduceExecutorLocal<Integer[], Integer, Integer>();
		
		
		int nrOfchunks = Runtime.getRuntime().availableProcessors();
		System.out.println(nrOfchunks);
		Collection<Integer[]> chunks = new ArrayList<Integer[]>();
		
		while(nrOfchunks-->0) {
			Random random = new Random();
			Integer[] data = new Integer[100000];
			for(int i = 0; i<data.length; i++) {
				data[i] = random.nextInt(4);
			}
			chunks.add(data);
		}
		
		System.out.println(Thread.currentThread().toString()+ " says: \"Starting single core computation\"");
		long start = System.currentTimeMillis();
		int sum = 0;
		for(Integer[] chunk : chunks ) {
			for( int i : chunk ) {
				sum += i;
			}
		}
		long singleCoreDuration = System.currentTimeMillis() - start;
		System.out.println(Thread.currentThread().toString()+ " says: \"The single core computed sum is "+sum+" which was computed in "+singleCoreDuration+" ms");
		
		System.out.println(Thread.currentThread().toString()+ " says: \"Starting "+Runtime.getRuntime().availableProcessors()+" core computation\"");
		start = System.currentTimeMillis();
		int mapReduceResult = mr.mapReduce(chunks, this, this);
		long multiCoreDuration = System.currentTimeMillis() - start;
		System.out.println(Thread.currentThread().toString()+ " says: \"The "+Runtime.getRuntime().availableProcessors()+" core computed sum is "+mapReduceResult+" which was computed in "+multiCoreDuration+" ms");
		
		Assert.assertEquals(sum, mapReduceResult);
		
	}
	

	@Override
	public Reducer<Integer, Integer> createReducer() {
		return new Reducer<Integer, Integer>() {
			@Override
			public Integer reduce(Collection<Integer> input) {
				int sum = 0;
				for(int i : input )
					sum += i;
				return sum;
			}
		};
	}

	@Override
	public Mapper<Integer[], Integer> createMapper() {
		return new Mapper<Integer[], Integer>() {
			@Override
			public Integer map(Integer[] input) {
				
				System.out.println(Thread.currentThread().toString()+ " says: \"Counting "+input.length+" integers\"");
				int intermediate = 0;
				for(int i : input)
					intermediate += i;
						
				System.out.println(Thread.currentThread().toString()+ " says: \"Returning intermediate result of "+intermediate+"\"");
				return intermediate;
			}
		};
	}

}
