package com.rapidminer.operator.preprocessing.transformation;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import static junit.framework.Assert.*;

import com.rapidminer.operator.learner.associations.Item;
import com.rapidminer.subspace.Interval;
import com.rapidminer.subspace.ItemParser;

public class ItemParserTest {
	
	public static final String DEFAULT_ATTRIBUTE_NAME = "att";
	
	public static final String LEFT_INFTY_RIGHT_NEG = "[- - -9.2]";
	public static final String LEFT_INFTY_RIGHT_POS = "[- - 9.2]";

	public static final String LEFT_NEG_RIGHT_NEG = "[-2.0 - -1.7]";
	public static final String LEFT_NEG_RIGHT_POS = "[-2.0 - 1.7]";
	public static final String LEFT_NEG_RIGHT_INFTY = "[-3.1 - ]";
	
	public static final String LEFT_POS_RIGHT_POS = "[3.7 - 4.0]";
	public static final String LEFT_POS_RIGHT_INFTY = "[3.1 - ]";
	
	public static final Map<String,Interval> OTHER_CASES = new HashMap<String, Interval>();
	
	private ItemParser parser;
	
	@Before
	public void setUp() throws Exception {
		parser = new ItemParser();
		
		OTHER_CASES.put("[- - -9.2]", new Interval(DEFAULT_ATTRIBUTE_NAME, Double.NEGATIVE_INFINITY, -9.2));
	}
	
	@Test
	public void leftInftyRightNeg() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_INFTY_RIGHT_NEG));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(Double.NEGATIVE_INFINITY, result.getLeft());
		assertEquals(-9.2, result.getRight());
	}
	
	@Test
	public void leftInftyRightPos() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_INFTY_RIGHT_POS));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(Double.NEGATIVE_INFINITY, result.getLeft());
		assertEquals(9.2, result.getRight());
	}
	
	@Test
	public void leftNegRightNeg() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_NEG_RIGHT_NEG));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(-2.0, result.getLeft());
		assertEquals(-1.7, result.getRight());
	}
	
	@Test
	public void leftNegRightPos() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_NEG_RIGHT_POS));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(-2.0, result.getLeft());
		assertEquals(1.7, result.getRight());
	}
	
	@Test
	public void leftNegRightInfty() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_NEG_RIGHT_INFTY));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(-3.1, result.getLeft());
		assertEquals(Double.POSITIVE_INFINITY, result.getRight());
	}
	
	@Test
	public void leftPosRightPos() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_POS_RIGHT_POS));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(3.7, result.getLeft());
		assertEquals(4.0, result.getRight());
	}
	
	@Test
	public void leftPosRightInfty() throws ParseException {
		Interval result = parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + LEFT_POS_RIGHT_INFTY));
		assertEquals(DEFAULT_ATTRIBUTE_NAME, result.getAttributeName());
		assertEquals(3.1, result.getLeft());
		assertEquals(Double.POSITIVE_INFINITY, result.getRight());
	}
	
	@Test
	public void otherCases() throws ParseException {
		for(String intervalString : OTHER_CASES.keySet() ) {
			assertEquals(OTHER_CASES.get(intervalString), parser.parse(new StringItem(DEFAULT_ATTRIBUTE_NAME + ItemParser.SEPERATOR + intervalString)));
		}
	}
	
	public class StringItem implements Item {
		private static final long serialVersionUID = 1L;
		private final String name;
		public StringItem(String name) {
			this.name = name;
		}
		@Override
		public String toString() {
			return name;
		}
		public int compareTo(Item o) {return 0;}
		public int getFrequency() {return 0;}
		public void increaseFrequency() {}
		public void increaseFrequency(int value) {}
		
	}

}
